unit PDSL.Environment;

interface

uses
  Generics.Collections, PDSL.Expressions;

type
  TEnvironment = class(TInterfacedObject, IEnvironment)
  private
    FEnv: TDictionary<string, IExpression>;
  public
    constructor Create;
    destructor Destroy; override;

    function GetValue(const AName: string): IExpression;
    function SetValue(const AName: string; AExpr: IExpression): IEnvironment;
  end;

implementation

{ TEnvironment }

constructor TEnvironment.Create;
begin
  inherited;
  FEnv := TDictionary<string, IExpression>.Create;
end;

destructor TEnvironment.Destroy;
begin
  FEnv.Free;
  inherited;
end;

function TEnvironment.GetValue(const AName: string): IExpression;
begin
  if not FEnv.TryGetValue(AName, Result) then
    raise EExprException.CreateFmt('Variable is not declared: %s', [AName]);
end;

function TEnvironment.SetValue(const AName: string; AExpr: IExpression): IEnvironment;
var
  EnvPair: TPair<string, IExpression>;
  NewEnv: TEnvironment;
begin
  NewEnv := TEnvironment.Create;
  for EnvPair in FEnv do
    NewEnv.FEnv.Add(EnvPair.Key, EnvPair.Value);
  NewEnv.FEnv.AddOrSetValue(AName, AExpr);

  Result := NewEnv;
end;

end.
